#include "pic.h"
#include "HW_Config.h"

void init_hw_p16f1827(void)
{

    OSCCON = 0b01101000;    // bit7: SPLLEN = OFF
                            // bit6-3: 4MHz
                            // bit 1-0: Clock determined by Config
    //OPTION_REG = 0b01110001;  //bit6 INTEDG: 1 = INT on rising edge
    OPTION_REG = 0b00110001;    // bit7 nRAPU: 0 = enable
                                // bit6 INTEDG: 0 = INT on falling edge
                                // bit5 T0CS:   0 = internal instruction cycle clock
                                //              1 = Transition on RA2/T0CKI pin
                                // bit4 T0SE: don't care
                                // bit3 PSA: 0 = Prescaler is assigned to the Timer0 module
                                // bit2-0 PS2:PS0: 001 = 1:4

    CCP1SEL = 0;    //CCP1/P1A function is on RB3

    // IO PORT
    ANSELA = 0;
    ANSELB = 0;
    LATA = 0x00;
    LATB = 0xFF;
    TRISA = 0b00100000;
    TRISB = 0x11110000;   // All output
    
    // Interruput on change
    IOCBP = 0;  //INTERRUPT-ON-CHANGE PORTA POSITIVE EDGE REGISTER
    IOCBN = 0b11110000; //INTERRUPT-ON-CHANGE PORTA NEGATIVE EDGE REGISTER
                        // active START/RA5, COURSE/RA2, GOAL/RA0

    //TMR4 for 7seg x 3dig
    T4CON = 0b00000010;
        //bit 7 Unimplemented: Read as ?0?
        //bit 6-3 TxOUTPS<3:0>: Timerx Output Postscaler Select bits
        //  0000 = 1:1 Postscaler
        //bit 2 TMRxON: Timerx On bit
        //bit 1-0 TxCKPS<1:0>: Timer2-type Clock Prescale Select bits
        //     10 = Prescaler is 16
    PR4 = 250;  // TMR4 period = 1usec * 16 * 250 = 4 msec

    //TMR2 for sound PWM
    T2CON = 0b00000010;
        //bit 7 Unimplemented: Read as ?0?
        //bit 6-3 TxOUTPS<3:0>: Timerx Output Postscaler Select bits
        //  0000 = 1:1 Postscaler
        //bit 2 TMRxON: Timerx On bit
        //bit 1-0 TxCKPS<1:0>: Timer2-type Clock Prescale Select bits
        //     10 = Prescaler is 16

    //CCP1
    CCP1CON = 0b00001100;
        //bit 7-6
        //If CCPxM<3:2> = 11:
        //  00 = Single output; PxA modulated; PxB, PxC, PxD assigned as port pins
        //bit 5-4
        //  DCxB<1:0>: PWM Duty Cycle Least Significant bits
        //bit 3-0
        //CCPxM<3:0>: ECCPx Mode Select bits
        //  11xx =PWM mode

    TMR2IE = 0;
    
    CCPTMRS = 0b00000000;
        //bit 1-0 C1TSEL<1:0>: CCP1 Timer Selection
        //       00 =CCP1 is based off Timer 2 in PWM Mode



}
