/*
 * File:   main.c
 * Author: family
 *
 * Created on 2013/02/07
 */


#include "xc.h"

#include "7seg.h"
#include "HW_Config.h"
#include "sound.h"

#define _XTAL_FREQ 4000000

//#define T10KS

#ifdef T10KS
#define TLIMIT 10000
#else
#define TLIMIT 1000
#endif

//XC8 configuration
#pragma config FOSC = INTOSC // INTOSCIO oscillator:
#pragma config MCLRE = OFF // MCLR pin function is RA3
#pragma config PWRTE = ON // PWRT enabled
#pragma config WDTE = OFF // WDT disabled
#pragma config PLLEN = OFF //
#pragma config BOREN = OFF // BOR enabled
#pragma config BORV = LO // BOR enabled
#pragma config CLKOUTEN = OFF
#pragma config LVP = OFF


__EEPROM_DATA(0xE7, 0x03, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF);  // high score 0x3E7 = 999

unsigned char mask_goal, mask_start;
unsigned char flg_ioc;
unsigned int t_sec; //, t_sec2;
unsigned char t_sec10, sec10_count;
unsigned int best_score;

unsigned char t4m_count;

void goto_bed(void);
void display_score(int score);
void display_best();
void clear_best();
void read_best();

void delay_100ms(void)
{
    __delay_ms(100);
}

void delay_500ms(void)
{
    __delay_ms(500);
}

void delay_sec(char s){
    int i;
    for(i=0; i<s*2; i++){
        delay_500ms();
    }
}

int main(void)
{
    unsigned char i;
    unsigned int j;
    INIT_HW();

    /*
    LATA = S_1;
    LATB = dig_lat_bit[2];
    while(1);
*/

    set7seg_off();

    start7seg();

    /*
    for(j=0; j<1000; j++){
        t4m_count = 0;
        set7seg_int(j);
        i = j % 12;
        sound_on(note_ary[i]);
        while(t4m_count < 50); // 200ms
        sound_off();
        while(t4m_count < 250); // 200ms
    }
    //set7seg_int(123);
    //set7seg_char(S_1, S_2, S_3);
    while(1);
    */
    
    i = 0;
    while(!SWCH)
    {
        delay_100ms();
        if(++i > 30)
            display_best();
    }
    delay_100ms();

    while(1){
        set7seg_char(S__, S__, S__);

        IOCBF = 0;
        flg_ioc = 0;
        INTCON = 0b11001000;

        i = 0;
        while(1){
            delay_100ms();
            if(!START){
                mask_goal = MASK_GOAL;
                mask_start = MASK_START;
                break;
            }
            if(!GOAL){
                mask_goal = MASK_START;
                mask_start = MASK_GOAL;
                break;
            }
            if(++i > 250 || (flg_ioc & MASK_SW) != 0)
                goto_bed();
        }

        set7seg_char(S_o, S_o_up, S_o);   // display "0"

        IOCBF = 0;
        flg_ioc = 0;
        INTCON = 0b11001000;
            //  bit7: GIE = 1;
            //  bit6: PEIE = 1;
            //  bit4: INTE = 1;
            //  bit3: IOCIE = 1;
            //  bit1: INTF = 0;
            //  bit0: IOCIF = 0;

        // wait for start
        i = 0;
        do{
            flg_ioc = 0;
            IOCIE = 1;
            delay_100ms();
            if(++i > 250 || (flg_ioc & MASK_SW) != 0)
                goto_bed();
        } while((flg_ioc & mask_start) != 0 || (PORT_SG & mask_start) == 0);

        //start
        set7seg_off();  // display off
        delay_500ms();
        if((flg_ioc & mask_start) != 0)
            break;

        IOCIF = 0;
        IOCBF = 0;
        flg_ioc = 0;
        IOCIE = 1;

        TMR1 = 0;
        TMR1IE = 0;
        TMR1ON = 1;

//        for(t_sec=2; t_sec<2000; t_sec++){  // priod of t_sec = 500msec
        t_sec10 = 0;
        for(t_sec = sec10_count = 1; t_sec<TLIMIT; t_sec++, sec10_count++){  // priod of t_sec = 500msec
            t4m_count = 0;
            if(sec10_count > 9){
                if(++t_sec10 > 9){
                    t_sec10 = 0;
                }
                sec10_count = 0;
            }
            sound_on(note_ary[t_sec10]);
            set7seg_int(t_sec);
            while(t4m_count < 50 && flg_ioc == 0); // 200ms
            if(flg_ioc)
                break;
            sound_off();
            while(t4m_count < 250&& flg_ioc == 0); // 1000ms
            if(flg_ioc)
                break;
        }   // End of for(t_sec)

        if(t_sec == TLIMIT){
            flg_ioc = MASK_COURSE;
        }

        //INTCON = 0;
        if((flg_ioc & mask_goal) != 0){
            // goal
            set7seg_char(S_G, S_0, S_L);
            sound_on(NOTE5C);

            delay_sec(1);
            read_best();
            sound_off();
            //t_sec2 = t_sec;


            for(i=0; i<5 || (PORT_SG & MASK_SG) != MASK_SG; i<0x0F? i++:i){
                set7seg_int(t_sec);
                delay_500ms();
                set7seg_off();
                __delay_ms(200);
            }
            //while(!(GOAL & START));
            if(t_sec < best_score){
                eeprom_write(0, (unsigned char)t_sec);
                eeprom_write(1, (unsigned char)(t_sec >> 8));
                set7seg_char(S_B, S_5, S_t);
                sound_pkmn();
            }

        }
        else if((flg_ioc & MASK_COURSE) != 0){
            //miss
            set7seg_char(S__, S__, S__);
            sound_on(NOTE4C);
            delay_sec(2);
            sound_off();
        }
        else if(flg_ioc == 0 || (flg_ioc & MASK_SW) != 0){
            //switch pressed
            goto_bed();
        }
    }   // End while(1)
     

    return 0;
}

static void interrupt isr(void)
{
    if(TMR4IF){
        LAT_SEG7_DIG |= MASK_SEG7_DIG;
        LAT_SEG7_SEG = dig_num[flg_dig];
        LAT_SEG7_DIG &= dig_lat_bit[flg_dig];
        flg_dig++;
        if(flg_dig > 2) flg_dig = 0;
        t4m_count++;
        TMR4IF = 0;
    }

    if(IOCIF){
        flg_ioc = IOCBF;
        IOCIE = 0;
        IOCIF = 0;
        IOCBF = 0;
    }
}




void goto_bed(void){
    stop7seg();

    sound_off();
    delay_100ms();
    while(!SWCH);
    delay_100ms();
    IOCBN = MASK_SW;
    IOCBF = 0;
    IOCIF = 0;
    IOCIE = 1;
    SLEEP();
    NOP();
    delay_100ms();
    asm("RESET");

}


void display_best(void){
    unsigned char j = 0;
    set7seg_char(S_B, S_5, S_t);
    while(!SWCH){
        delay_100ms();
        if(++j>50){
            //Clear best
            set7seg_char(S_C, S_L, S_r);
            sound_on_10ms(12, 100);
            eeprom_write(0, 0xE7);
            eeprom_write(1, 0x03);
            while(!SWCH);
            break;
        }
    }
    read_best();
    set7seg_int(best_score);
    delay_sec(3);
}

void read_best(){
    best_score = eeprom_read(1);
    best_score <<= 8;
    best_score += eeprom_read(0);
}

void display_score(int score){
    set7seg_int(score);
    delay_sec(3);
}
 